package metaobject;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

public class MetaObjectManagerPanel extends JPanel {
	private MetaObjectTableModel model = null;
	private JTable table = null;
	private MetaObject[] row;
	private Class metaObjectClass;
	private Runnable dataBinding = null;
	private Runnable refreshing = null;
	private JPanel topPanel;
	private Validatable validatable;
	private String title;
	private boolean isReloading;

	public MetaObjectManagerPanel(String title, Class metaObjectClass) {
		super();
		this.setLayout(new BorderLayout());
		this.title = title;
		this.metaObjectClass = metaObjectClass;
		try {
			this.model =
				new MetaObjectTableModel(
					(MetaObject) Class
						.forName(this.metaObjectClass.getName())
						.newInstance());
			this.table = new JTable(this.model);
			table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			table.getTableHeader().setReorderingAllowed(false);
			this.add(new JScrollPane(table), BorderLayout.CENTER);

			this.reload();

			this.topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			final JButton addButton = new JButton("add");
			addButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						if (validatable == null || validatable.isAddable()) {
							successAction(
								new MetaObjectFormDialog(
									MetaObjectManagerPanel.this,
									(MetaObject) Class
										.forName(
											MetaObjectManagerPanel
												.this
												.metaObjectClass
												.getName())
										.newInstance(),
									MetaObjectManagerPanel.this.title,
									false),
								"Success to " + addButton.getText());
						}
					} catch (Exception ex) {
					}
				}
			});
			topPanel.add(addButton);

			final JButton editButton = new JButton("edit");
			editButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					final int selectedRow = table.getSelectedRow();
					if (selectedRow >= 0) {
						waitReloding();
						if (validatable == null || validatable.isEditalbe()) {
							successAction(
								new MetaObjectFormDialog(
									MetaObjectManagerPanel.this,
									row[selectedRow],
									MetaObjectManagerPanel.this.title,
									true),
								"Success to " + editButton.getText());
						}
					}
				}
			});
			topPanel.add(editButton);

			final JButton removeButton = new JButton("remove");
			removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					final int selectedRow = table.getSelectedRow();
					if (selectedRow >= 0) {
						waitReloding();
						if (validatable == null || validatable.isRemovable()) {
							XMLObject.remove(row[selectedRow]);
							successAction(
								null,
								"Success to " + removeButton.getText());
						}
					}
				}
			});
			topPanel.add(removeButton);

			this.add(topPanel, BorderLayout.NORTH);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void waitReloding() {
		while (isReloading) {
			try {
				Thread.sleep(500);
			} catch (Exception e) {
			}
		}
	}

	public void successAction(MetaObjectFormDialog dialog, String message) {
		boolean isOK = false;
		if (dialog != null) {
			if (dialog.getStat() == MetaObjectFormDialog.OK) {
				isOK = true;
			}
		} else {
			isOK = true;
		}

		if (isOK) {
			JOptionPane.showMessageDialog(
				UIUtilities.getFrame(this),
				message,
				title,
				JOptionPane.INFORMATION_MESSAGE);
			reload();
		}
	}

	public void failAction(String message) {
		JOptionPane.showMessageDialog(
			UIUtilities.getFrame(this),
			message,
			title,
			JOptionPane.ERROR_MESSAGE);
	}

	public void reload() {
		new Thread() {
			public void run() {
				isReloading = true;
				try {
					row = XMLObject.read(metaObjectClass);
					if (dataBinding != null) {
						dataBinding.run();
					}
					model.setDataVector(row);
					UIUtilities.fixColumnSize(table);
				} catch (Exception e) {
					e.printStackTrace();
				}
				isReloading = false;
			}
		}
		.start();
	}

	public Runnable getDataBinding() {
		return dataBinding;
	}

	public void setDataBinding(Runnable binding) {
		dataBinding = binding;
	}
	public Class getMetaObjectClass() {
		return metaObjectClass;
	}

	public MetaObjectTableModel getModel() {
		return model;
	}

	public MetaObject[] getRow() {
		return row;
	}

	public JTable getTable() {
		return table;
	}

	public void setRow(MetaObject[] objects) {
		row = objects;
	}

	public void setSubComponent(JComponent[] components) {
		for (int i = 0; components != null && i < components.length; i++) {
			topPanel.add(components[i]);
		}
		topPanel.revalidate();
		topPanel.repaint();
	}

	public Runnable getRefreshing() {
		return refreshing;
	}

	public void setRefreshing(Runnable runnable) {
		if (runnable != null) {
			refreshing = runnable;
			if (refreshing != null) {
				new Thread(refreshing).start();
			}
		}
	}

	public void setValidatable(Validatable validatable) {
		this.validatable = validatable;
	}

	public static void main(String[] args) {
		JFrame f = new JFrame();
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setContentPane(new MetaObjectManagerPanel("Test", Database.class));
		f.setSize(new Dimension(600, 600));
		f.setVisible(true);
	}

	public boolean isReloading() {
		return isReloading;
	}
}
